@@ -98,6 +98,7 @@
               key_params/0,
               digest_type/0,
               issuer_name/0,
+              bad_cert_reason/0,
               cert/0,
               combined_cert/0,
               cert_id/0,
@@ -159,7 +160,7 @@
 -type oid()                  :: tuple().
 -type cert_id()              :: {SerialNr::integer(), issuer_name()} .
 -type issuer_name()          :: {rdnSequence,[[#'AttributeTypeAndValue'{}]]} .
--type bad_cert_reason()      :: cert_expired | invalid_issuer | invalid_signature | name_not_permitted | missing_basic_constraint | invalid_key_usage | {revoked, crl_reason()} | atom().
+-type bad_cert_reason()      :: cert_expired | invalid_issuer | invalid_signature | name_not_permitted | missing_basic_constraint | invalid_key_usage | {key_usage_mismatch, term()} | {revoked, crl_reason()} | atom().
 
 -type combined_cert()        :: #cert{}.
 -type cert()                 :: der_cert() | otp_cert().
@@ -221,7 +222,7 @@ pem_entry_decode({'SubjectPublicKeyInfo', Der, _}) ->
             {params, DssParams} = der_decode('DSAParams', Params),
             {der_decode(KeyType, Key0), DssParams};
         'ECPoint' ->
-	    ECCParams = der_decode('EcpkParameters', Params),
+            ECCParams = ec_decode_params(AlgId, Params),
             {#'ECPoint'{point = Key0}, ECCParams}
     end;
 pem_entry_decode({Asn1Type, Der, not_encrypted}) when is_atom(Asn1Type),
@@ -1446,6 +1447,12 @@ cacerts_clear() ->
 %%--------------------------------------------------------------------
 %%% Internal functions
 %%--------------------------------------------------------------------
+ec_decode_params(AlgId, _) when AlgId == ?'id-Ed25519';
+                                AlgId == ?'id-Ed448' ->
+    {namedCurve, AlgId};
+ec_decode_params(_, Params) ->
+    der_decode('EcpkParameters', Params).
+
 default_options([]) ->
     [{rsa_padding, rsa_pkcs1_padding}];
 default_options(Opts) ->
